package paysafecard;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import org.w3c.dom.NodeList;

public class PSCSOAPClient {

    private SOAPMessage soapMessage;
    private SOAPMessage soapResponse;
    public String sresult;

    public PSCSOAPClient() throws Exception {
        MessageFactory messageFactory = MessageFactory.newInstance();
        soapMessage = messageFactory.createMessage();
        SOAPPart soapPart = soapMessage.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();
        envelope.addNamespaceDeclaration("urn", "urn:pscservice");
    }

    public void doCall(String url) {
        try {
            SOAPConnectionFactory soapConnectionFactory = SOAPConnectionFactory.newInstance();
            SOAPConnection soapConnection = soapConnectionFactory.createConnection();

            SOAPMessage response = soapConnection.call(soapMessage, url);
            soapResponse = response;

            ByteArrayOutputStream out = new ByteArrayOutputStream();
            soapResponse.writeTo(out);
            sresult = out.toString();

            soapConnection.close();
        } catch (Exception e) {
            //Log.addLog("error_soap_call", "doCall", url, "_null_", "error");
        }
    }

    public void addElement(String name, HashMap<String, String> kvp, HashMap<String, ArrayList<HashMap<String, String>>> childElements) throws Exception {
        SOAPPart soapPart = soapMessage.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();
        SOAPBody soapBody = envelope.getBody();
        SOAPElement soapBodyElement = soapBody.addChildElement(name, "urn");
        if (!name.equals("getPayoutState")) {
            SOAPElement customer = soapBodyElement.addChildElement("customerDetailsBasic", "urn");
            for (Map.Entry<String, String> entry : kvp.entrySet()) {
                if (entry.getKey().equals("firstName") || entry.getKey().equals("lastName") || entry.getKey().equals("dateOfBirth")) {
                    customer.addChildElement(entry.getKey(), "urn").addTextNode(entry.getValue());
                } else {
                    soapBodyElement.addChildElement(entry.getKey(), "urn").addTextNode(entry.getValue());
                }
            }
        } else {
            for (Map.Entry<String, String> entry : kvp.entrySet()) {
                soapBodyElement.addChildElement(entry.getKey(), "urn").addTextNode(entry.getValue());
            }
        }

        if (childElements.isEmpty() == false) {
            for (Entry<String, ArrayList<HashMap<String, String>>> entry : childElements.entrySet()) {
                for (int i = 0; i < childElements.get(entry.getKey()).size(); i++) {
                    SOAPElement element = soapBodyElement.addChildElement(entry.getKey(), "urn");
                    HashMap<String, String> elementMap = childElements.get(entry.getKey()).get(i);
                    element.addChildElement("key", "urn").addTextNode(elementMap.get("key"));
                    element.addChildElement("value", "urn").addTextNode(elementMap.get("value"));
                }
            }
        }
        soapMessage.saveChanges();
    }

    public String getElement(String name) throws Exception {
        NodeList nl = soapResponse.getSOAPBody().getElementsByTagName(name);
        return nl.item(0).getTextContent();
    }

}
