package paysafecard;

import static java.lang.System.out;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Random;

public class Payout {

    boolean live = false;
    private HashMap<String, String> data = new HashMap<String, String>();
    private HashMap<String, String> response = new HashMap<String, String>();
    private String url;
    private String error;

    public Payout(String uname, String password, boolean live) {
        data.put("username", uname);
        data.put("password", password);
        url = live ? "https://soa.paysafecard.com/psc/services/PscService?wsdl" : "https://soatest.paysafecard.com/psc/services/PscService?wsdl";
    }

    public void setTransaction(String amount, String currency, String ptid, String utcOffset) {
        data.put("ptid", ptid.isEmpty() ? generatePtid() : ptid);
        data.put("amount", amount.replace(",", "."));
        data.put("currency", currency.toUpperCase());
        data.put("utcOffset", utcOffset);
    }

    public void setCustomer(String merchantClientId, String customerId, String customerIdType) {
        data.put("customerId", customerId);
        data.put("merchantClientId", merchantClientId);
        data.put("customerIdType", customerIdType);
    }

    public void setCustomerDetails(String firstName, String lastName, String dateOfBirth) {
        data.put("firstName", firstName);
        data.put("lastName", lastName);
        data.put("dateOfBirth", dateOfBirth);
    }

    public void payoutDo(String validationOnly) throws Exception {
        HashMap<String, String> params = new HashMap<String, String>();
        params.put("firstName", data.get("firstName"));
        params.put("lastName", data.get("lastName"));
        params.put("dateOfBirth", data.get("dateOfBirth"));
        params.put("username", data.get("username"));
        params.put("password", data.get("password"));
        params.put("customerId", data.get("customerId"));
        params.put("merchantClientId", data.get("merchantClientId"));
        params.put("customerIdType", data.get("customerIdType"));
        params.put("ptid", data.get("ptid"));
        params.put("amount", data.get("amount"));
        params.put("currency", data.get("currency"));
        params.put("utcOffset", data.get("utcOffset"));
        params.put("validationOnly", validationOnly);

        PSCSOAPClient connection = new PSCSOAPClient();
        connection.addElement("payout", params, new HashMap<String, ArrayList<HashMap<String, String>>>());

        connection.doCall(url);

        response.put("errorCodeDescription", connection.getElement("ns1:errorCodeDescription"));
        response.put("errorCode", connection.getElement("ns1:errorCode"));
        response.put("resultCode", connection.getElement("ns1:resultCode"));
        response.put("soap", connection.sresult.toString());
    }

    public HashMap getPayoutState() throws Exception {
        HashMap<String, String> responscodes = new HashMap<String, String>();
        HashMap<String, String> params = new HashMap<String, String>();
        params.put("username", data.get("username"));
        params.put("password", data.get("password"));
        PSCSOAPClient connection = new PSCSOAPClient();
        connection.addElement("getPayoutState", params, new HashMap<String, ArrayList<HashMap<String, String>>>());

        connection.doCall(url);

        response.put("soap", connection.sresult.toString());

        responscodes.put("totalPayoutAmount", connection.getElement("ns1:totalPayoutAmount"));
        responscodes.put("totalPaymentAmount", connection.getElement("ns1:totalPaymentAmount"));
        responscodes.put("totalPayoutBalance", connection.getElement("ns1:totalPayoutBalance"));
        responscodes.put("dailyPayoutLimit", connection.getElement("ns1:dailyPayoutLimit"));
        responscodes.put("creditLine", connection.getElement("ns1:creditLine"));
        responscodes.put("dailyPayoutAmount", connection.getElement("ns1:dailyPayoutAmount"));
        responscodes.put("dailyPayoutBalance", connection.getElement("ns1:dailyPayoutBalance"));

        return responscodes;
    }

    public String getResultCode() {
        return response.get("resultCode");
    }

    public String getSoapResponse() {
        return response.get("soap");
    }

    public String getErrorCode() {
        return response.get("errorCode");
    }

    public String getErrorCodeDescription() {
        return response.get("errorCodeDescription");
    }

    private String generatePtid() {
        Random rand = new Random();
        int randomNum = rand.nextInt((9999 - 1) + 1) + 1;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyymmdd-Hms");
        String date = sdf.format(new Date());

        String rtext = "test_" + date + "_" + randomNum;

        return rtext;
    }

    public String getError(String resultcode) {
        switch (resultcode) {
            case "3167_20":
            case "3167_40":
            case "3167":
            case "3170_20":
            case "3170_40":
            case "3170":
            case "3194_20":
            case "3194_40":
            case "3194":
            case "3197":
            case "3198":
                return "Unfortunately there has been a problem executing your request. Please check your e-mail inbox for further information from paysafecard on how to solve the issue and try again.";
            case "3162":
                return "Unfortunately, no my paysafecard account exists under the e-mail address you have entered. Please check the address for a typing error. If you do not have a my paysafecard account, you can <a href=\"https://mypins.paysafecard.com/customer-registration/StartRegistration.xhtml\" id=\"paysafecard_link\">register for one online now for free</a>.";
            case "3195_20":
            case "3195_40":
            case "3195":
                return "The personal details associated with your my paysafecard account do not match the details of this account. Please check the first names, surnames and dates of birth entered in both accounts and request the payout again.";
            default:
                return "Sorry, the payout was not completed successfully! There was no money transfered. Please contact our customer support: support@company.com";
        }
    }
}
